library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity top is
    Port ( clk : in STD_LOGIC;
           rst : in STD_LOGIC;
           s : in UNSIGNED(7 downto 0); --precio producto maximo 255
           a : in UNSIGNED(7 downto 0); --valor de moneda moneda max 255 (1,2,50,20,10,5)
           c : in STD_LOGIC; --pulso para introducir moneda
           d : out STD_LOGIC; --pulso de salida valida
           cambio : out UNSIGNED(8 downto 0); --valor devuelto
           d_cambio : out STD_LOGIC; --seal de devolucion
           error : out STD_LOGIC);
end top;

architecture Behavioral of top is
    type STATE is (Init, Waitt, Add, Disp);
    signal PSTATE: STATE := Init; --Estado inicializado a Init.
    signal tot: unsigned(8 downto 0); --Suma del saldo introducido.
    signal Nmonedas : unsigned (3 downto 0) := "0000"; --Contador de monedas introducidas.
    signal cuenta_tiempo : std_logic :='0'; --Inicia cuenta de tiempo
    signal tiempo : unsigned (4 downto 0) :="00000"; --Contador del nmero de ciclos de inactividad.
begin
    process (rst, clk)
    begin
        if (rst= '1') then --Se resetea al estado inicial.
            PSTATE <= Init;
            tot<=(others =>'0');
            error<='0';
        elsif (clk='1' and clk'EVENT) then
            case PSTATE is
                when Init =>
                    d<='0';
                    tot<=(others =>'0');
                    error<='0';
                    d_cambio<='0';
                    cuenta_tiempo<='0';
                    tiempo<=(others => '0');
                    cambio<=(others => '0');
                    Nmonedas<=(others =>'0');
                    PSTATE <=Waitt;
                when Waitt =>
                    if cuenta_tiempo='1' then --Se incrementa el nmero de ciclos
                        if tiempo <30 then    --si no se han alcanzado los 30 ciclos
                            tiempo<=tiempo+1; --de inactividad.
                        else
                            error<='1';       --Si se alcanzan los 30 ciclos de
                            PSTATE<=Init;     --inactividad, se activa la seal
                        end if;               --de error y se resetea el sistema.
                    end if;
                    if Nmonedas < 10 then
                        if (c='1') then
                            if(a=5 or a=10 or a=20 or a=50 or a=100 or a=200) then
                                Nmonedas<=Nmonedas+1; --Si se introduce una moneda vlida,
                                cuenta_tiempo<='1';   --se incrementa el nmero de monedas
                                tiempo<=(others =>'0');--introducidas,se resetea el perodo
                                PSTATE <= Add;         --de inactividad y activa contador de tiempo.
                            else
                                error<='1';            --Si la moneda introducida es incorrecta,
                            end if;                    --se activa la seal de error.
                                         
                        elsif (tot>=s) then --Si se introduce un saldo igual o mayor al precio
                            PSTATE <= Disp; --del refresco, la mquina dispensa.
                        end if;
                    else
                        error<='1';   --Si se introducen ms de 10 monedas,
                        d_cambio<='1';--se activa la seal de error, la
                        cambio<=tot;  --mquina devuelve el saldo y se resetea
                        PSTATE<=Init; --el sistema.
                    end if;
                when Add =>
                    tot<=tot+a;        --suma saldo
                    PSTATE <= Waitt;
                when Disp =>
                    d<='1';            --Cuando la mquina dispensa el producto,
                    d_cambio<='0';     --se activa la seal de devolucin del
                    cambio<=tot-s;     --cambio, la seal de dispensar el producto,
                    PSTATE<=Init;      -- el cambio y se resetea el sistema.
                end case;
       end if;
   end process;
end Behavioral;